// hlavickove subory *******************************************************
#include "../includes.h"

// konstruktor *************************************************************
CMath3d::CMath3d()
{

}

// destruktor **************************************************************
CMath3d::~CMath3d()
{

}

// kolizia svetelneho luca a trojuholnika **********************************
bool CMath3d::ray_tri_collision(vec3 P,vec3 L,vec3 N,float D,vec4 CN0,vec4 CN1,vec4 CN2)
{
 // vektor P -> L (bod -> poloha svetelneho zdroja)
 v0=L-P; v0.normalize();
 v1=P-L; v1.normalize();

 // skalarny sucin normaly trojuholnika a svetelneho luca
 v2=N;
 a=v2.dot(v0);
 b=v2.dot(v1);

 // v pripade, ak je luc kolmy na normalu trojuholnika, nedoslo ku kolizii
 if (a==0.0f) return(false);

 // projekcia bodu do roviny definovanej trojuholnikom
 a=-(N.x*P.x+N.y*P.y+N.z*P.z+D)/a;
 b=-(N.x*L.x+N.y*L.y+N.z*L.z+D)/b;

 // ak je rovina, ktoru tvori trojuholnik medzi bodom a svetelnym zdrojom,
 // je potrebne otestovat ci priesecnik luca a roviny lezi v trojuholniku
 if ((a>0.0f)&&(b>0.0f)) {
  // priesecnik luca a roviny tvorenej trojuholnikom
  v1=P+v0*a;
  // ak bod lezi v trojuholniku, doslo ku kolizii
  return(point_in_triangle(v1,CN0,CN1,CN2));
 }

 // nedoslo ku kolizii
 return(false);
}

// priesecnik vektora a roviny *********************************************
bool CMath3d::vec_plane_intersection(vec3 A,vec3 B,vec3 N,float D)
{
 // vektor A -> B
 v0=B-A; v0.normalize();

 // skalarny sucin vektora a normaly roviny
 a=v0.dot(N);

 // v pripade, ak je vektor kolmy na normalu roviny,
 // alebo ak je odvrateny od roviny, neexistuje priesecnik
 if (a>=0.0f) return(false);

 // projekcia bodu do roviny
 a=-(N.x*A.x+N.y*A.y+N.z*A.z+D)/a;
 // priesecnik vektora a roviny
 v1=A+v0*a;

 return(true);
}

// ziskanie priesecnika luca a trojuholnika ********************************
vec3 CMath3d::get_ipoint(void)
{
 return(v1);
}

// test polohy bodu a trojuholnika *****************************************
bool CMath3d::point_in_triangle(vec3 P,vec4 CN0,vec4 CN1,vec4 CN2)
{
 // ak bod lezi za stenou trojuholnika => nelezi v trojuholniku
 if ((P.x*CN0.x+P.y*CN0.y+P.z*CN0.z+CN0.w)>0.0f) return(false);
 if ((P.x*CN1.x+P.y*CN1.y+P.z*CN1.z+CN1.w)>0.0f) return(false);
 if ((P.x*CN2.x+P.y*CN2.y+P.z*CN2.z+CN2.w)>0.0f) return(false);

 // bod lezi pred vsetkymi troma stenami => lezi v trojuholniku
 return(true);
}

// kolizia bodu so stranami trojuholnika ***********************************
bool CMath3d::edge_collision(vec3 *P,vec3 *S,vec3 A,vec3 B,vec3 C,float R)
{
 // 1. strana
 v0=point_on_line(A,B,*P); v1=*P-v0;
 if (v1.magnitude()<R) { v1.normalize(); *P=(v0+v1*R); *S=v0; return(true); }
 // 2. strana
 v0=point_on_line(B,C,*P); v1=*P-v0;
 if (v1.magnitude()<R) { v1.normalize(); *P=(v0+v1*R); *S=v0; return(true); }
 // 3. strana
 v0=point_on_line(C,A,*P); v1=*P-v0;
 if (v1.magnitude()<R) { v1.normalize(); *P=(v0+v1*R); *S=v0; return(true); }

 // nedoslo ku kolizii
 return(false);
}

// vypocet polohy bodu na ciare A-B, ktory je najblizsie k bodu C **********
vec3 CMath3d::point_on_line(vec3 A,vec3 B,vec3 C)
{
 v0=C-A; // vektor A->C
 v1=B-A; // vektor A->B
 v2=v1;  // kopia vektora A->B

 v1.normalize(); // normalizacia vektora A->B

 // skalarny sucin vektorov A->C a A->B
 a=v0.dot(v1);

 // bod C lezi pred zaciatkom ciary => najblizsim bodom k C je A
 if (a<=0.0f) return(A);

 b=v2.magnitude(); // dlzka vektora A->B

 // bod C lezi za koncom ciary => najblizsim bodom k C je B
 if (a>=b) return(B);

 // vypocet polohy bodu na ciare A-B
 return(A+v1*a);
}

// orezanie stran 4-uholnika podla stran 3-uholnika ************************
int CMath3d::quad_tri_clip(vec3 *V,vec2 *T,vec4 *TRI)
{
 was_clip=false;

 vnum=4;       // na zaciatku n-uholnik = 4-uholnik
 V[vnum]=V[0]; // posledny vrchol + 1 = prvy vrchol
 T[vnum]=T[0]; // posledne texturovacie suradnice + 1 = prve texturovacie suradnice

 // pre vsetky strany 3-uholnika
 loopl(3) {
  ai=0; // pocet vrcholov v cielovom poli

  // pre vsetky vrcholy n-uholnika
  loopk(vnum) {
   // vzdialenost koncovych vrcholov strany n-uholnika od roviny, ktora vytvara 2 polpriestory
   a=V[k].x*TRI[l].x+V[k].y*TRI[l].y+V[k].z*TRI[l].z+TRI[l].w;
   b=V[k+1].x*TRI[l].x+V[k+1].y*TRI[l].y+V[k+1].z*TRI[l].z+TRI[l].w;

   if (a*b<0.0f) { // koncove vrcholy su na opacnych stranach roviny
    if (a<=0.0f) { p[ai]=V[k]; q[ai]=T[k]; ai++; }

    // vypocet priesecnika strany n-uholnika a roviny, ktora vytvara 2 polpriestory
    v0=V[k+1]-V[k]; b=v0.magnitude(); v0.normalize();
    // skalarny sucin normaly roviny a vektora
    v1.set(TRI[l].x,TRI[l].y,TRI[l].z);
    a=v0.dot(v1);
    // projekcia vrcholu do roviny
    a=-(v1.x*V[k].x+v1.y*V[k].y+v1.z*V[k].z+TRI[l].w)/a;
    // priesecnik vektora a roviny
    v0=V[k]+v0*a;
    p[ai]=v0; // novy vrchol

    v0=v0-V[k]; a=v0.magnitude();
    a=a/b;
    v3=T[k+1]-T[k];
    q[ai]=T[k]+v3*a; // nove texturovacie suradnice

    ai++; // bol pridany novy vrchol a nove texturovacie suradnice

    was_clip=true;
   }
   else { // koncove vrcholy su na rovnakych stranach roviny
    if (a<=0.0f) { p[ai]=V[k]; q[ai]=T[k]; ai++; }
   }
  } // pre vsetky vrcholy n-uholnika

  // presun novych vrcholov do povodneho pola
  loopi(ai) { V[i]=p[i]; T[i]=q[i]; }
  V[ai]=V[0]; // posledny vrchol + 1 = prvy vrchol
  T[ai]=T[0]; // posledne texturovacie suradnice + 1 = prve texturovacie suradnice

  vnum=ai; // aktualny pocet vrcholov n-uholnika
 } // pre vsetky strany 3-uholnika

 return(ai);
}

// orezanie stran 3-uholnika podla stran 3D hranola ************************
int CMath3d::tri_box_clip(vec3 *V,vec4 *P)
{
 was_clip=false;

 vnum=3;       // na zaciatku n-uholnik = 3-uholnik
 V[vnum]=V[0]; // posledny vrchol + 1 = prvy vrchol

 // pre vsetky strany 3D hranola
 loopl(6) {
  ai=0; // pocet vrcholov v cielovom poli

  // pre vsetky vrcholy n-uholnika
  loopk(vnum) {
   // vzdialenost koncovych vrcholov strany n-uholnika od roviny, ktora vytvara 2 polpriestory
   a=V[k].x*P[l].x+V[k].y*P[l].y+V[k].z*P[l].z+P[l].w;
   b=V[k+1].x*P[l].x+V[k+1].y*P[l].y+V[k+1].z*P[l].z+P[l].w;

   if (a*b<0.0f) { // koncove vrcholy su na opacnych stranach roviny
    if (a<=0.0f) { p[ai]=V[k]; ai++; }

    // vypocet priesecnika strany n-uholnika a roviny, ktora vytvara 2 polpriestory
    v0=V[k+1]-V[k]; b=v0.magnitude(); v0.normalize();
    // skalarny sucin normaly roviny a vektora
    v1.set(P[l].x,P[l].y,P[l].z);
    a=v0.dot(v1);
    // projekcia vrcholu do roviny
    a=-(v1.x*V[k].x+v1.y*V[k].y+v1.z*V[k].z+P[l].w)/a;
    // priesecnik vektora a roviny
    v0=V[k]+v0*a;
    p[ai]=v0; // novy vrchol

    ai++; // bol pridany novy vrchol a nove texturovacie suradnice

    was_clip=true;
   }
   else { // koncove vrcholy su na rovnakych stranach roviny
    if (a<=0.0f) { p[ai]=V[k]; ai++; }
   }
  } // pre vsetky vrcholy n-uholnika

  // presun novych vrcholov do povodneho pola
  loopi(ai) V[i]=p[i];
  V[ai]=V[0]; // posledny vrchol + 1 = prvy vrchol

  vnum=ai; // aktualny pocet vrcholov n-uholnika

 } // pre vsetky strany 3D hranola

 return(ai);
}

// test ci trojuholnik zasahuje do 3D hranola ******************************
bool CMath3d::tri_in_box(vec3 A,vec3 B,vec3 C,vec4 *P)
{
 // pre vsetkych 6 rovin, ktore su definovane stranami 3D hranola
 loopk(6) {
  // ak su vsetky 3 vrcholy trojuholnika pred rovinou, trojuholnik
  // nezasahuje do 3D hranola
  // A,B,C - vrcholy trojuholnika
  // P - parametre rovnice roviny (x,y,z = normala, w = d)
  ai=0;
  if ((P[k].x*A.x+P[k].y*A.y+P[k].z*A.z+P[k].w)>0.0f) ai++;
  if ((P[k].x*B.x+P[k].y*B.y+P[k].z*B.z+P[k].w)>0.0f) ai++;
  if ((P[k].x*C.x+P[k].y*C.y+P[k].z*C.z+P[k].w)>0.0f) ai++;
  if (ai==3) return(false);
 }

 return(true);
}
