// hlavickove subory *******************************************************
#include "includes.h"
// vlastny .h subor
#include "frontend.h"

// pouzitie ShaderProgramov
#include "common/shader.h"

// konstruktor *************************************************************
CFrontend::CFrontend()
{

}

// destruktor **************************************************************
CFrontend::~CFrontend()
{

}

// inicializacia vykreslovania
void CFrontend::init(void)
{
 MVoffset[0]=0.8f;                     // 0.7 0.0 0.0 0.0
 MVoffset[5]=0.6f;                     // 0.0 0.6 0.0 0.0
 MVoffset[10]=0.0f;                    // 0.0 0.0 0.0 0.0
 MVoffset[12]=0.5f; MVoffset[13]=0.5f; // 0.5 0.5 0.0 1.0

 // nacitanie projection matice - je konstantna
 glGetFloatv(GL_PROJECTION_MATRIX,(GLfloat *) &PRmat);
 PRmat=MVoffset*PRmat;
}

// vykreslenie vsetkych trojuholnikov **************************************
void CFrontend::tris_draw(GLfloat FL,vec3 AP)
{
 // identifikator zakladnej textury a lightmapy
 base_tex_id=0; lm_tex_id=0;

 // so shadermi ------------------------------------------------------------
 if (glextMT&&glextVP&&glextFP) {
  // pouzije sa multitexturing:
  // 1.TU - zakladna textura (base)
  // 2.TU - normalova mapa (normalmap)
  // 3.TU - svetelna mapa (lightmapa)
  // 4.TU - svietidlo (flashlight)

  // 1. texturovacia jednotka
  glActiveTexture_(GL_TEXTURE0_ARB); glEnable(GL_TEXTURE_2D);
  // 2. texturovacia jednotka
  glActiveTexture_(GL_TEXTURE1_ARB); glEnable(GL_TEXTURE_2D);
  // 3. texturovacia jednotka
  glActiveTexture_(GL_TEXTURE2_ARB); glEnable(GL_TEXTURE_2D);
  // 4. texturovacia jednotka
  glActiveTexture_(GL_TEXTURE3_ARB); glEnable(GL_TEXTURE_2D);
  // flashlight
  glBindTexture(GL_TEXTURE_2D,texture.tex_id[tid[2]]);

  // uprava texturovacej matice pre flashlight
  glPushMatrix();
  glTranslatef(0.2f,-0.2f,0.0f);
  glGetFloatv(GL_MODELVIEW_MATRIX,(GLfloat *) &MVmat);
  glPopMatrix();

  Tmat=PRmat*MVmat; // Tmat=MVoffset*PRmat*MVmat

  glMatrixMode(GL_TEXTURE);
  glPushMatrix();
  glLoadMatrixf(Tmat.get());
  glMatrixMode(GL_MODELVIEW);

  // pouziju sa polia atributov vrcholov:
  // 0 - suradnice vrcholu
  // 8 - 1. texturovacie suradnice
  // 9 - 2. texturovacie suradnice
  glEnableVertexAttribArray_(0);
  glEnableVertexAttribArray_(8);
  glEnableVertexAttribArray_(9);

  // vyber 2 najblizsich svetelnych zdrojov - zatial iba pokus
  l0=0; l1=1;
  d0=(lights[l0].p+AP).magnitude(); d1=(lights[l1].p+AP).magnitude();
  // pre vsetky svetelne zdroje
  loopvk(lights) {
   if (k<2) continue;
   d2=(lights[k].p+AP).magnitude();
   if (d2<d0) { d0=d2; l0=k; }
   else if (d2<d1) { d1=d2; l1=k; }
  } // pre vsetky svetelne zdroje

  // pre vsetky trojuholniky
  loopvk(tris) {

   // ak je potrebne vymenit zakladnu texturu
   if (base_tex_id!=tris[k].tex1id) {
    // base
    glActiveTexture_(GL_TEXTURE0_ARB);
    glBindTexture(GL_TEXTURE_2D,tris[k].tex1id);
    // normalmap
    if (tris[k].tex2id!=0) {
     glActiveTexture_(GL_TEXTURE1_ARB);
     glBindTexture(GL_TEXTURE_2D,tris[k].tex2id);
    }
    base_tex_id=tris[k].tex1id;
   }
   // ak je potrebne vymenit lightmapu
   if (lm_tex_id!=tris[k].lmid) {
    // lightmap
    glActiveTexture_(GL_TEXTURE2_ARB);
    glBindTexture(GL_TEXTURE_2D,tris[k].lmid);
    lm_tex_id=tris[k].lmid;
   }

   glVertexAttribPointer_(0,3,GL_FLOAT,0,0,tris[k].v); // vertex
   glVertexAttribPointer_(8,2,GL_FLOAT,0,0,tris[k].t); // texcoord
   glVertexAttribPointer_(9,2,GL_FLOAT,0,0,tris[k].l); // lm texcoord

   // trojuholnik ma normalmapu
   if (tris[k].tex2id!=0) {
    // normalmap & flashlight shader
    nm_fl_vp.PRGenable(); nm_fl_vp.PRGbind();
    nm_fl_fp.PRGenable(); nm_fl_fp.PRGbind();

    // TNB
    nm_fl_vp.PRGlocal_param(0,vec4(tris[k].tangent.x,tris[k].tangent.y,tris[k].tangent.z,0.0f));
    nm_fl_vp.PRGlocal_param(1,vec4(tris[k].binormal.x,tris[k].binormal.y,tris[k].binormal.z,0.0f));
    nm_fl_vp.PRGlocal_param(2,vec4(tris[k].n.x,tris[k].n.y,tris[k].n.z,0.0f));
    // poloha 2 najblizsich svetelnych zdrojov
    nm_fl_vp.PRGlocal_param(3,vec4(lights[l0].p.x,lights[l0].p.y,lights[l0].p.z,0.0f));
    nm_fl_vp.PRGlocal_param(4,vec4(lights[l1].p.x,lights[l1].p.y,lights[l1].p.z,0.0f));
    // farba svetiel, flashlight
    nm_fl_fp.PRGlocal_param(0,lights[0].c);
    nm_fl_fp.PRGlocal_param(1,vec4(FL,0.0f,0.0f,0.0f));
/*
    // GLSL shader
    if (glextGLSL) {
     glsl_shader.GLSLon();
     glsl_shader.GLSLuniformF("T",tris[k].tangent.x,tris[k].tangent.y,tris[k].tangent.z,0.0f);
     glsl_shader.GLSLuniformF("B",tris[k].binormal.x,tris[k].binormal.y,tris[k].binormal.z,0.0f);
     glsl_shader.GLSLuniformF("N",tris[k].n.x,tris[k].n.y,tris[k].n.z,0.0f);
     glsl_shader.GLSLuniformF("L1",lights[l0].p.x,lights[l0].p.y,lights[l0].p.z,0.0f);
     glsl_shader.GLSLuniformF("L2",lights[l1].p.x,lights[l1].p.y,lights[l1].p.z,0.0f);
     glsl_shader.GLSLuniformF("C",-AP.x,-AP.y,-AP.z,0.0f);
    }
*/
    glDrawArrays(GL_TRIANGLES,0,3);

    nm_fl_fp.PRGdisable();
    nm_fl_vp.PRGdisable();

    //if (glextGLSL) glsl_shader.GLSLoff();
   }
   // trojuholnik nema normalmapu
   else {
    // flashlight shader
    fl_vp.PRGenable(); fl_vp.PRGbind();
    fl_fp.PRGenable(); fl_fp.PRGbind();
    fl_fp.PRGlocal_param(0,vec4(FL,0.0f,0.0f,0.0f));

    glDrawArrays(GL_TRIANGLES,0,3);

    fl_fp.PRGdisable(); fl_vp.PRGdisable();
   }

  } // pre vsetky trojuholniky

  glDisableVertexAttribArray_(0);
  glDisableVertexAttribArray_(8);
  glDisableVertexAttribArray_(9);

  // zrusenie zmien texturovacej matice
  glMatrixMode(GL_TEXTURE);
  glPopMatrix();
  glMatrixMode(GL_MODELVIEW);

  // 4. texturovacia jednotka
  glActiveTexture_(GL_TEXTURE3_ARB);
  glDisable(GL_TEXTURE_2D);
  // 3. texturovacia jednotka
  glActiveTexture_(GL_TEXTURE2_ARB);
  glDisable(GL_TEXTURE_2D);
  // 2. texturovacia jednotka
  glActiveTexture_(GL_TEXTURE1_ARB);
  glDisable(GL_TEXTURE_2D);
  // 1. texturovacia jednotka
  glActiveTexture_(GL_TEXTURE0_ARB);
 } // so shadermi
 // bez shaderov -----------------------------------------------------------
 else {
  // ulozenie potrebnych atributov
  glPushAttrib(GL_TEXTURE_BIT);
  //glEnable(GL_TEXTURE_2D);
  glDisable(GL_TEXTURE_2D);

  glEnableClientState(GL_VERTEX_ARRAY);
  glEnableClientState(GL_TEXTURE_COORD_ARRAY);

  // pre vsetky trojuholniky
  loopvk(tris) {

   // ak je potrebne vymenit zakladnu texturu
   if (base_tex_id!=tris[k].tex1id) {
    // base
    glBindTexture(GL_TEXTURE_2D,tris[k].tex1id);
    base_tex_id=tris[k].tex1id;
   }

   glVertexPointer(3,GL_FLOAT,0,tris[k].v);   // vertex
   glTexCoordPointer(2,GL_FLOAT,0,tris[k].t); // texcoord

   glDrawArrays(GL_TRIANGLES,0,3);
/*
   // INFO: vykreslenie normal, koliznych normal, TB vektorov
   glPushAttrib(GL_TEXTURE_BIT);
   glDisable(GL_TEXTURE_2D);

   glColor4f(1.0f,0.0f,0.0f,1.0f); // normaly - cervenou
   glBegin(GL_LINES);
    glVertex3f(tris[k].v[0].x,tris[k].v[0].y,tris[k].v[0].z);
    glVertex3f(tris[k].v[0].x+tris[k].vn[0].x,tris[k].v[0].y+tris[k].vn[0].y,tris[k].v[0].z+tris[k].vn[0].z);

    glVertex3f(tris[k].v[1].x,tris[k].v[1].y,tris[k].v[1].z);
    glVertex3f(tris[k].v[1].x+tris[k].vn[1].x,tris[k].v[1].y+tris[k].vn[1].y,tris[k].v[1].z+tris[k].vn[1].z);

    glVertex3f(tris[k].v[2].x,tris[k].v[2].y,tris[k].v[2].z);
    glVertex3f(tris[k].v[2].x+tris[k].vn[2].x,tris[k].v[2].y+tris[k].vn[2].y,tris[k].v[2].z+tris[k].vn[2].z);
   glEnd();

   glColor4f(1.0f,1.0f,0.0f,1.0f); // kolizne normaly - zltou
   glBegin(GL_LINES);
    glVertex3f(tris[k].v[0].x,tris[k].v[0].y,tris[k].v[0].z);
    glVertex3f(tris[k].v[0].x+tris[k].cn[0].x,tris[k].v[0].y+tris[k].cn[0].y,tris[k].v[0].z+tris[k].cn[0].z);

    glVertex3f(tris[k].v[1].x,tris[k].v[1].y,tris[k].v[1].z);
    glVertex3f(tris[k].v[1].x+tris[k].cn[1].x,tris[k].v[1].y+tris[k].cn[1].y,tris[k].v[1].z+tris[k].cn[1].z);

    glVertex3f(tris[k].v[2].x,tris[k].v[2].y,tris[k].v[2].z);
    glVertex3f(tris[k].v[2].x+tris[k].cn[2].x,tris[k].v[2].y+tris[k].cn[2].y,tris[k].v[2].z+tris[k].cn[2].z);
   glEnd();

   glColor4f(0.0f,1.0f,0.0f,1.0f); // tangent - zelenou
   glBegin(GL_LINES);
    glVertex3f(tris[k].v[0].x,tris[k].v[0].y,tris[k].v[0].z);
    glVertex3f(tris[k].v[0].x+tris[k].tangent.x,tris[k].v[0].y+tris[k].tangent.y,tris[k].v[0].z+tris[k].tangent.z);
   glEnd();

   glColor4f(0.0f,0.0f,1.0f,1.0f); // binormal - modrou
   glBegin(GL_LINES);
    glVertex3f(tris[k].v[0].x,tris[k].v[0].y,tris[k].v[0].z);
    glVertex3f(tris[k].v[0].x+tris[k].binormal.x,tris[k].v[0].y+tris[k].binormal.y,tris[k].v[0].z+tris[k].binormal.z);
   glEnd();

   // standardna farba - biela
   glColor4f(1.0f,1.0f,1.0f,1.0f);

   glPopAttrib();
*/
  } // pre vsetky trojuholniky

  glDisableClientState(GL_VERTEX_ARRAY);
  glDisableClientState(GL_TEXTURE_COORD_ARRAY);

  glPopAttrib(); // vyber ulozenych atributov - TEXTURE
 } // bez shaderov
}
