#ifndef _INTERFACE_H_
#define _INTERFACE_H_

// triedy pre pracu s jednotlivymi modulmi Enginu
#include "common/avatar.h"
#include "common/text.h"
#include "backend.h"
#include "frontend.h"

// historia fps
#define FPSHISTORY 20

struct Smaterial { // struktura OpenGL materialu
 GLfloat amb[4],dif[4],spe[4],amb_dif[4],emi[4];
 GLfloat shi,transp,sillum[4];
};

struct Slight { // struktura OpenGL svetla
 GLfloat amb[4],dif[4],spe[4];
 GLfloat pos[4],spot[3];
};

// trieda premennych a funkcii (objektov) pre pracu s OpenGL
class CInterface {
public:
 CInterface();  // konstruktor
 ~CInterface(); // destruktor

 // zmena stavov OpenGL stavoveho stroja
 bool fog_onoff;         // zap/vyp hmly
 bool lighting_onoff;    // zap/vyp osvetlenia
 bool help_onoff;        // zap/vyp vypisu helpu
 bool pause_onoff;       // zap/vyp casovo zavisleho pohybu objektov
 bool normals_onoff;     // zap/vyp vykreslovania normal
 bool polygon_onoff;     // zap/vyp vykreslovania hran
 float flashlight_onoff; // zap/vyp svietidla

 // pocet snimkov za sekundu
 GLuint actFPS;

 CAvatar avatar;     // instancia triedy CAvatar - praca s kamerou
 CText text;         // instancia triedy CText - praca s textom
 CBackend backend;   // instancia triedy CBackend - praca s OpenGL trojuholnikmi
 CFrontend frontend; // instancia triedy CFrontend - rendering sceny

 void resettime(void);  // nastartovanie casovaca
 void updatetime(void); // aktualizacia casovaca

 // inicializacia OpenGL - nastavenie parametrov OpenGL sceny
 int OGL_initialize(void);
 void OGL_draw(void); // vykreslenie OpenGL sceny
 void OGL_idle(void); // casovo zavisla uprava objektov, resp. sceny

 void change_polygon_mode(void); // vyplnene trojuholniky/iba hrany
 void change_flashlight_onoff(void); // zap/vyp svietidla
private:
 // farba hmly = farba pozadia
 GLfloat fogcolor[4];

 Smaterial material[1]; // pouzite materialy
 Slight light[1];       // pouzite svetla

 GLint OGL_state[2]; // pre zistovanie stavov OpenGL stavoveho stroja

 // aktualny a povodny casovy okamih (ms), pocet uplynutych milisekund
 GLuint actTime,oldTime,millis;
 // pomocne premenne
 GLint fpsdelay,fpserror;
 // pocitadlo snimkov
 GLuint frames;
 // historia fps
 GLuint fpspos,fpstotal,fpshistory[FPSHISTORY];
 // minimalny a maximalny pocet snimkov za sekundu
 GLuint minfps,maxfps;

 // velkost zameriavacieho kriza
 GLfloat crosshairSize;

 // aktualny pocet fps, minimalny a maximalny pocet fps
 void getfpshistory(GLuint &ACT,GLuint &MIN,GLuint &MAX);

 void drawhud(void); // vykreslenie head-up obrazovky
};

#endif
